/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.fleet.parser.config;

import java.io.FileNotFoundException;

import com.sunlabs.archsim.fleet.nameTable.SourceDestinationNameTable;

import junit.framework.TestCase;
import junit.textui.TestRunner;

/**
 * Test for parser for fleet aliases (sources and destinations).
 * 
 * @author ib27688
 *
 */
public class ConfigParserTest extends TestCase {

	private String path= ("src/" 
			+ ConfigParserTest.class.getPackage().getName().replace('.', '/'))
			+ "/";
	
	// used to run tests "manually
	public static void main(String []args) {
        // run the tests
        TestRunner.run(ConfigParserTest.class);		
	}
	
	protected void setUp() throws Exception {
		super.setUp();
		SourceDestinationNameTable.getInstance().clear();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	public void testNoFile() {
		ConfigParser configParser= new ConfigParser();
		boolean exceptionOK= false;
		try {
			configParser.parse("NoSuchFile");
		}
		catch (Exception e) {
			exceptionOK= (e instanceof FileNotFoundException);
		}
		assert(exceptionOK);
	}
	
	public void testGoodConfig() {
		ConfigParser configParser= new ConfigParser();
		boolean noException= true;
		try {
			configParser.parse(path+"GoodConfig.fc");		}
		catch (Exception e) {
			noException= false;
			System.out.println("Exception message: " + e.getMessage());
		}
		assert(noException);
		try {
			SourceDestinationNameTable map= SourceDestinationNameTable.getInstance();
			assertEquals(0, map.getSourceAddress("ZERO"));
			assertEquals(1, map.getSourceAddress("ONE"));
			assertEquals(2, map.getSourceAddress("TWO"));
			assertEquals(0, map.getDestinationAddress("DZERO"));
			assertEquals(1, map.getDestinationAddress("DONE"));
			assertEquals(22, map.getDestinationAddress("TWO"));
			assertEquals(4, map.getSourceAddress("FOUR.port"));
		}
		catch (Exception e) {
			assert(false);
		}
		
	}

} // ConfigParserTest
