/*
 * Created on November 9, 2005, 3:37 PM
 *
 */

package com.sunlabs.archsim.fleet.ships.duplicator;

import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.fleet.common.OutputFunction;
import com.sunlabs.archsim.gasp.Function;
// import com.sunlabs.archsim.gasp.State;
// import com.sunlabs.archsim.gasp.Terminal;
// import com.sunlabs.archsim.gasp.terminals.OutputTerminal;
import com.sunlabs.archsim.globals.Globals;
// import com.sunlabs.archsim.simulationBase.Command;

// import java.util.Iterator;

/**
 *
 * @author mh193489
 */
public class DuplicatorAction extends Action {
    private Delay adderDelay= DuplicatorDefaults.ADDER_DELAY_DEF;
    
    Function outFunction= new DupplicateData();
    Function ackInFunction= new AckFunction();
    
    /** references to terminals attached to this action */
    InputTerminal in;

    public DuplicatorAction(String name, CompositeEntity parent) {
        super(name, parent);
        outputFunctions.put(DuplicatorDefaults.OUT1, outFunction);
        outputFunctions.put(DuplicatorDefaults.OUT2, outFunction);
        outputFunctions.put(DuplicatorDefaults.ACK_IN, ackInFunction);
        
        
    } // DuplicatorAction
    
    public void setDelay(Delay d) {
        adderDelay= (Delay)d.clone();
        outFunction.setDelay(adderDelay);
        ackInFunction.setDelay(adderDelay);
    }
    
    public boolean selfCheck() {
        boolean OK= true;
        boolean check;
        
        check= (this.getTerminal(DuplicatorDefaults.IN) != null);
        OK= OK && check;
        if (!check) {
            director.error("Action " + getAlias() + ", id= " + getID()
            + ", does not have attached terminal in");
        } else {
            in= (InputTerminal)this.getTerminal(DuplicatorDefaults.IN);
        }
        
        check= (this.getTerminal(DuplicatorDefaults.ACK_IN) != null);
        OK= OK && check;
        if (!check) {
            director.error("Action " + getAlias() + ", id= " + getID()
            + ", does not have attached terminal ackIn");
        }
        
        check= (this.getTerminal(DuplicatorDefaults.OUT1) != null);
        OK= OK && check;
        if (!check) {
            director.error("Action " + getAlias() + ", id= " + getID()
            + ", does not have attached terminal Out1");
        }
        
        check= (this.getTerminal(DuplicatorDefaults.ACK_OUT1) != null);
        OK= OK && check;
        if (!check) {
            director.error("Action " + getAlias() + ", id= " + getID()
            + ", does not have attached terminal ackOut1");
        }
        
        check= (this.getTerminal(DuplicatorDefaults.OUT2) != null);
        OK= OK && check;
        if (!check) {
            director.error("Action " + getAlias() + ", id= " + getID()
            + ", does not have attached terminal Out2");
        }
        
        check= (this.getTerminal(DuplicatorDefaults.ACK_OUT2) != null);
        OK= OK && check;
        if (!check) {
            director.error("Action " + getAlias() + ", id= " + getID()
            + ", does not have attached terminal ackOut2");
        }
        
        // also run the self check on the superclass
        check= super.selfCheck();
        OK= OK && check;
        
        return OK;
    }
    
        
    private class DupplicateData extends Function {
        
        public DupplicateData() {
            super(DuplicatorDefaults.ADD_DATA);
            functionDelay= adderDelay;
        }
        
        public Object value() {
            // create a copy of the data from input terminal
            Data data= (Data)((Payload)in.getData()).data.clone();
            return data;            
        } // value
    } // AddData
    
    private class AckFunction extends OutputFunction {
        
        /** Global settings */
        protected Globals globals= Globals.getInstance();
        
        public AckFunction() {
            this("AckFunction");
        }
        
        public AckFunction(String name) {
            super(name);
            Integer fDelay= globals.intValue(DuplicatorDefaults.ACK_DELAY);
            functionDelay= (fDelay != null)
            ? new Delay(fDelay.intValue())
            : DuplicatorDefaults.ACK_DELAY_DEF;
        }
        
        
        public Data value() {
            return null;
        } // value
        
    } // AckFunction
}


